@@ -48,7 +48,6 @@
 #include "RequestGroup.h"
 #include "download_helper.h"
 #include "util.h"
-#include "RequestGroupMan.h"
 #include "fmt.h"
 #include "RpcRequest.h"
 #include "PieceStorage.h"
@@ -148,7 +147,7 @@ const std::string KEY_NUM_ACTIVE = "numActive";
 namespace {
 SharedHandle<ValueBase> createGIDResponse(a2_gid_t gid)
 {
-  return String::g(util::itos(gid));
+  return String::g(GroupId::toHex(gid));
 }
 } // namespace
 
@@ -185,7 +184,19 @@ namespace {
 a2_gid_t str2Gid(const String* str)
 {
   assert(str);
-  return util::parseLLInt(str->s());
+  if(str->s().size() > sizeof(a2_gid_t)*2) {
+    throw DL_ABORT_EX(fmt("Invalid GID %s", str->s().c_str()));
+  }
+  a2_gid_t n;
+  switch(GroupId::expandUnique(n, str->s().c_str())) {
+  case GroupId::ERR_NOT_UNIQUE:
+    throw DL_ABORT_EX(fmt("GID %s is not unique", str->s().c_str()));
+  case GroupId::ERR_NOT_FOUND:
+    throw DL_ABORT_EX(fmt("GID %s is not found", str->s().c_str()));
+  case GroupId::ERR_INVALID:
+    throw DL_ABORT_EX(fmt("Invalid GID %s", str->s().c_str()));
+  }
+  return n;
 }
 } // namespace
 
@@ -273,19 +284,23 @@ SharedHandle<ValueBase> AddTorrentRpcMethod::process
   bool posGiven = checkPosParam(posParam);
   size_t pos = posGiven ? posParam->i() : 0;
 
-  std::string filename = util::applyDir
-    (requestOption->get(PREF_DIR), getHexSha1(torrentParam->s())+".torrent");
-  std::vector<SharedHandle<RequestGroup> > result;
-  // Save uploaded data in order to save this download in
-  // --save-session file.
-  if(util::saveAs(filename, torrentParam->s(), true)) {
-    A2_LOG_INFO(fmt("Uploaded torrent data was saved as %s", filename.c_str()));
-    requestOption->put(PREF_TORRENT_FILE, filename);
-  } else {
-    A2_LOG_INFO(fmt("Uploaded torrent data was not saved."
-                    " Failed to write file %s", filename.c_str()));
-    filename.clear();
+  std::string filename;
+  if(requestOption->getAsBool(PREF_RPC_SAVE_UPLOAD_METADATA)) {
+    filename = util::applyDir
+      (requestOption->get(PREF_DIR), getHexSha1(torrentParam->s())+".torrent");
+    // Save uploaded data in order to save this download in
+    // --save-session file.
+    if(util::saveAs(filename, torrentParam->s(), true)) {
+      A2_LOG_INFO(fmt("Uploaded torrent data was saved as %s",
+                      filename.c_str()));
+      requestOption->put(PREF_TORRENT_FILE, filename);
+    } else {
+      A2_LOG_INFO(fmt("Uploaded torrent data was not saved."
+                      " Failed to write file %s", filename.c_str()));
+      filename.clear();
+    }
   }
+  std::vector<SharedHandle<RequestGroup> > result;
   createRequestGroupForBitTorrent(result, requestOption, uris, filename,
                                   torrentParam->s());
 
@@ -320,22 +335,27 @@ SharedHandle<ValueBase> AddMetalinkRpcMethod::process
 
   std::vector<SharedHandle<RequestGroup> > result;
 #ifdef ENABLE_MESSAGE_DIGEST
-  // TODO RFC5854 Metalink has the extension .meta4 and Metalink
-  // Version 3 uses .metalink extension. We use .meta4 for both
-  // RFC5854 Metalink and Version 3. aria2 can detect which of which
-  // by reading content rather than extension.
-  std::string filename = util::applyDir
-    (requestOption->get(PREF_DIR), getHexSha1(metalinkParam->s())+".meta4");
-  // Save uploaded data in order to save this download in
-  // --save-session file.
-  if(util::saveAs(filename, metalinkParam->s(), true)) {
-    A2_LOG_INFO(fmt("Uploaded metalink data was saved as %s",
-                    filename.c_str()));
-    requestOption->put(PREF_METALINK_FILE, filename);
-    createRequestGroupForMetalink(result, requestOption);
+  std::string filename;
+  if(requestOption->getAsBool(PREF_RPC_SAVE_UPLOAD_METADATA)) {
+    // TODO RFC5854 Metalink has the extension .meta4 and Metalink
+    // Version 3 uses .metalink extension. We use .meta4 for both
+    // RFC5854 Metalink and Version 3. aria2 can detect which of which
+    // by reading content rather than extension.
+    filename = util::applyDir
+      (requestOption->get(PREF_DIR), getHexSha1(metalinkParam->s())+".meta4");
+    // Save uploaded data in order to save this download in
+    // --save-session file.
+    if(util::saveAs(filename, metalinkParam->s(), true)) {
+      A2_LOG_INFO(fmt("Uploaded metalink data was saved as %s",
+                      filename.c_str()));
+      requestOption->put(PREF_METALINK_FILE, filename);
+      createRequestGroupForMetalink(result, requestOption);
+    } else {
+      A2_LOG_INFO(fmt("Uploaded metalink data was not saved."
+                      " Failed to write file %s", filename.c_str()));
+      createRequestGroupForMetalink(result, requestOption, metalinkParam->s());
+    }
   } else {
-    A2_LOG_INFO(fmt("Uploaded metalink data was not saved."
-                    " Failed to write file %s", filename.c_str()));
     createRequestGroupForMetalink(result, requestOption, metalinkParam->s());
   }
 #else // !ENABLE_MESSAGE_DIGEST
@@ -350,11 +370,11 @@ SharedHandle<ValueBase> AddMetalinkRpcMethod::process
     }
     for(std::vector<SharedHandle<RequestGroup> >::const_iterator i =
           result.begin(), eoi = result.end(); i != eoi; ++i) {
-      gids->append(util::itos((*i)->getGID()));
+      gids->append(GroupId::toHex((*i)->getGID()));
     }
   }
   return gids;
-} 
+}
 #endif // ENABLE_METALINK
 
 namespace {
@@ -377,12 +397,13 @@ SharedHandle<ValueBase> removeDownload
       if(group->isDependencyResolved()) {
         e->getRequestGroupMan()->removeReservedGroup(gid);
       } else {
-        throw DL_ABORT_EX(fmt("GID#%" PRId64 " cannot be removed now", gid));
+        throw DL_ABORT_EX(fmt("GID#%s cannot be removed now",
+                              GroupId::toHex(gid).c_str()));
       }
     }
   } else {
-    throw DL_ABORT_EX(fmt("Active Download not found for GID#%" PRId64,
-                          gid));
+    throw DL_ABORT_EX(fmt("Active Download not found for GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   return createGIDResponse(gid);
 }
@@ -441,7 +462,8 @@ SharedHandle<ValueBase> pauseDownload
       return createGIDResponse(gid);
     }
   }
-  throw DL_ABORT_EX(fmt("GID#%" PRId64 " cannot be paused now", gid));
+  throw DL_ABORT_EX(fmt("GID#%s cannot be paused now",
+                        GroupId::toHex(gid).c_str()));
 }
 } // namespace
 
@@ -463,7 +485,7 @@ void pauseRequestGroups
 (InputIterator first, InputIterator last, bool reserved, bool forcePause)
 {
   for(; first != last; ++first) {
-    pauseRequestGroup(*first, reserved, forcePause);
+    pauseRequestGroup((*first).second, reserved, forcePause);
   }
 }
 } // namespace
@@ -472,10 +494,9 @@ namespace {
 SharedHandle<ValueBase> pauseAllDownloads
 (const RpcRequest& req, DownloadEngine* e, bool forcePause)
 {
-  const std::deque<SharedHandle<RequestGroup> >& groups =
-    e->getRequestGroupMan()->getRequestGroups();
+  const RequestGroupList& groups = e->getRequestGroupMan()->getRequestGroups();
   pauseRequestGroups(groups.begin(), groups.end(), false, forcePause);
-  const std::deque<SharedHandle<RequestGroup> >& reservedGroups =
+  const RequestGroupList& reservedGroups =
     e->getRequestGroupMan()->getReservedGroups();
   pauseRequestGroups(reservedGroups.begin(), reservedGroups.end(),
                      true, forcePause);
@@ -505,10 +526,11 @@ SharedHandle<ValueBase> UnpauseRpcMethod::process
   if(!group ||
      group->getState() != RequestGroup::STATE_WAITING ||
      !group->isPauseRequested()) {
-    throw DL_ABORT_EX(fmt("GID#%" PRId64 " cannot be unpaused now", gid));
+    throw DL_ABORT_EX(fmt("GID#%s cannot be unpaused now",
+                          GroupId::toHex(gid).c_str()));
   } else {
     group->setPauseRequested(false);
-    e->getRequestGroupMan()->requestQueueCheck();    
+    e->getRequestGroupMan()->requestQueueCheck();
   }
   return createGIDResponse(gid);
 }
@@ -516,12 +538,13 @@ SharedHandle<ValueBase> UnpauseRpcMethod::process
 SharedHandle<ValueBase> UnpauseAllRpcMethod::process
 (const RpcRequest& req, DownloadEngine* e)
 {
-  const std::deque<SharedHandle<RequestGroup> >& groups =
+  const RequestGroupList& groups =
     e->getRequestGroupMan()->getReservedGroups();
-  std::for_each(groups.begin(), groups.end(),
-                std::bind2nd(mem_fun_sh(&RequestGroup::setPauseRequested),
-                             false));
-  e->getRequestGroupMan()->requestQueueCheck();    
+  for(RequestGroupList::SeqType::const_iterator i = groups.begin(),
+        eoi = groups.end(); i != eoi; ++i) {
+    (*i).second->setPauseRequested(false);
+  }
+  e->getRequestGroupMan()->requestQueueCheck();
   return VLB_OK;
 }
 
@@ -630,7 +653,7 @@ void gatherProgressCommon
 {
   const SharedHandle<PieceStorage>& ps = group->getPieceStorage();
   if(requested_key(keys, KEY_GID)) {
-    entryDict->put(KEY_GID, util::itos(group->getGID()));
+    entryDict->put(KEY_GID, GroupId::toHex(group->getGID()).c_str());
   }
   if(requested_key(keys, KEY_TOTAL_LENGTH)) {
     // This is "filtered" total length if --select-file is used.
@@ -643,14 +666,14 @@ void gatherProgressCommon
   }
   TransferStat stat = group->calculateStat();
   if(requested_key(keys, KEY_DOWNLOAD_SPEED)) {
-    entryDict->put(KEY_DOWNLOAD_SPEED, util::itos(stat.getDownloadSpeed()));
+    entryDict->put(KEY_DOWNLOAD_SPEED, util::itos(stat.downloadSpeed));
   }
   if(requested_key(keys, KEY_UPLOAD_SPEED)) {
-    entryDict->put(KEY_UPLOAD_SPEED, util::itos(stat.getUploadSpeed()));
+    entryDict->put(KEY_UPLOAD_SPEED, util::itos(stat.uploadSpeed));
   }
   if(requested_key(keys, KEY_UPLOAD_LENGTH)) {
     entryDict->put
-      (KEY_UPLOAD_LENGTH, util::itos(stat.getAllTimeUploadLength()));
+      (KEY_UPLOAD_LENGTH, util::itos(stat.allTimeUploadLength));
   }
   if(requested_key(keys, KEY_CONNECTIONS)) {
     entryDict->put(KEY_CONNECTIONS, util::itos(group->getNumConnection()));
@@ -676,14 +699,14 @@ void gatherProgressCommon
       // The element is GID.
       for(std::vector<a2_gid_t>::const_iterator i = group->followedBy().begin(),
             eoi = group->followedBy().end(); i != eoi; ++i) {
-        list->append(util::itos(*i));
+        list->append(GroupId::toHex(*i));
       }
       entryDict->put(KEY_FOLLOWED_BY, list);
     }
   }
   if(requested_key(keys, KEY_BELONGS_TO)) {
     if(group->belongsTo()) {
-      entryDict->put(KEY_BELONGS_TO, util::itos(group->belongsTo()));
+      entryDict->put(KEY_BELONGS_TO, GroupId::toHex(group->belongsTo()));
     }
   }
   if(requested_key(keys, KEY_FILES)) {
@@ -752,8 +775,7 @@ void gatherProgressBitTorrent
     } else {
       const SharedHandle<PeerStorage>& peerStorage = btObject->peerStorage;
       assert(peerStorage);
-      std::vector<SharedHandle<Peer> > peers;
-      peerStorage->getActivePeers(peers);
+      const PeerSet& peers = peerStorage->getUsedPeers();
       entryDict->put(KEY_NUM_SEEDERS,
                      util::uitos(countSeeder(peers.begin(), peers.end())));
     }
@@ -765,10 +787,12 @@ namespace {
 void gatherPeer
 (const SharedHandle<List>& peers, const SharedHandle<PeerStorage>& ps)
 {
-  std::vector<SharedHandle<Peer> > activePeers;
-  ps->getActivePeers(activePeers);
-  for(std::vector<SharedHandle<Peer> >::const_iterator i =
-        activePeers.begin(), eoi = activePeers.end(); i != eoi; ++i) {
+  const PeerSet& usedPeers = ps->getUsedPeers();
+  for(PeerSet::const_iterator i = usedPeers.begin(), eoi = usedPeers.end();
+      i != eoi; ++i) {
+    if(!(*i)->isActive()) {
+      continue;
+    }
     SharedHandle<Dict> peerEntry = Dict::g();
     peerEntry->put(KEY_PEER_ID, util::torrentPercentEncode((*i)->getPeerId(),
                                                            PEER_ID_LENGTH));
@@ -782,9 +806,10 @@ void gatherPeer
                    util::toHex((*i)->getBitfield(), (*i)->getBitfieldLength()));
     peerEntry->put(KEY_AM_CHOKING, (*i)->amChoking()?VLB_TRUE:VLB_FALSE);
     peerEntry->put(KEY_PEER_CHOKING, (*i)->peerChoking()?VLB_TRUE:VLB_FALSE);
-    TransferStat stat = ps->getTransferStatFor(*i);
-    peerEntry->put(KEY_DOWNLOAD_SPEED, util::itos(stat.getDownloadSpeed()));
-    peerEntry->put(KEY_UPLOAD_SPEED, util::itos(stat.getUploadSpeed()));
+    peerEntry->put(KEY_DOWNLOAD_SPEED,
+                   util::itos((*i)->calculateDownloadSpeed()));
+    peerEntry->put(KEY_UPLOAD_SPEED,
+                   util::itos((*i)->calculateUploadSpeed()));
     peerEntry->put(KEY_SEEDER, (*i)->isSeeder()?VLB_TRUE:VLB_FALSE);
     peers->append(peerEntry);
   }
@@ -801,7 +826,7 @@ void gatherProgress
 {
   gatherProgressCommon(entryDict, group, keys);
 #ifdef ENABLE_BITTORRENT
-  if(group->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT)) {
+  if(group->getDownloadContext()->hasAttribute(CTX_ATTR_BT)) {
     SharedHandle<TorrentAttribute> torrentAttrs =
       bittorrent::getTorrentAttrs(group->getDownloadContext());
     const SharedHandle<BtObject>& btObject =
@@ -817,7 +842,7 @@ void gatherStoppedDownload
  const std::vector<std::string>& keys)
 {
   if(requested_key(keys, KEY_GID)) {
-    entryDict->put(KEY_GID, util::itos(ds->gid));
+    entryDict->put(KEY_GID, ds->gid->toHex());
   }
   if(requested_key(keys, KEY_ERROR_CODE)) {
     entryDict->put(KEY_ERROR_CODE, util::itos(static_cast<int>(ds->result)));
@@ -837,14 +862,14 @@ void gatherStoppedDownload
       // The element is GID.
       for(std::vector<a2_gid_t>::const_iterator i = ds->followedBy.begin(),
             eoi = ds->followedBy.end(); i != eoi; ++i) {
-        list->append(util::itos(*i));
+        list->append(GroupId::toHex(*i));
       }
       entryDict->put(KEY_FOLLOWED_BY, list);
     }
   }
   if(requested_key(keys, KEY_BELONGS_TO)) {
     if(ds->belongsTo) {
-      entryDict->put(KEY_BELONGS_TO, util::itos(ds->belongsTo));
+      entryDict->put(KEY_BELONGS_TO, GroupId::toHex(ds->belongsTo));
     }
   }
   if(requested_key(keys, KEY_FILES)) {
@@ -907,8 +932,8 @@ SharedHandle<ValueBase> GetFilesRpcMethod::process
     SharedHandle<DownloadResult> dr =
       e->getRequestGroupMan()->findDownloadResult(gid);
     if(!dr) {
-      throw DL_ABORT_EX(fmt("No file data is available for GID#%" PRId64 "",
-                            gid));
+      throw DL_ABORT_EX(fmt("No file data is available for GID#%s",
+                            GroupId::toHex(gid).c_str()));
     } else {
       createFileEntry(files, dr->fileEntries.begin(), dr->fileEntries.end(),
                       dr->totalLength, dr->pieceLength, dr->bitfield);
@@ -934,7 +959,8 @@ SharedHandle<ValueBase> GetUrisRpcMethod::process
   a2_gid_t gid = str2Gid(gidParam);
   SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX(fmt("No URI data is available for GID#%" PRId64, gid));
+    throw DL_ABORT_EX(fmt("No URI data is available for GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   SharedHandle<List> uriList = List::g();
   // TODO Current implementation just returns first FileEntry's URIs.
@@ -953,7 +979,8 @@ SharedHandle<ValueBase> GetPeersRpcMethod::process
   a2_gid_t gid = str2Gid(gidParam);
   SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX(fmt("No peer data is available for GID#%" PRId64, gid));
+    throw DL_ABORT_EX(fmt("No peer data is available for GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   SharedHandle<List> peers = List::g();
   const SharedHandle<BtObject>& btObject =
@@ -983,7 +1010,8 @@ SharedHandle<ValueBase> TellStatusRpcMethod::process
     SharedHandle<DownloadResult> ds =
       e->getRequestGroupMan()->findDownloadResult(gid);
     if(!ds) {
-      throw DL_ABORT_EX(fmt("No such download for GID#%" PRId64 "", gid));
+      throw DL_ABORT_EX(fmt("No such download for GID#%s",
+                            GroupId::toHex(gid).c_str()));
     }
     gatherStoppedDownload(entryDict, ds, keys);
   } else {
@@ -1010,21 +1038,20 @@ SharedHandle<ValueBase> TellActiveRpcMethod::process
   std::vector<std::string> keys;
   toStringList(std::back_inserter(keys), keysParam);
   SharedHandle<List> list = List::g();
-  const std::deque<SharedHandle<RequestGroup> >& groups =
-    e->getRequestGroupMan()->getRequestGroups();
-  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        groups.begin(), eoi = groups.end(); i != eoi; ++i) {
+  const RequestGroupList& groups = e->getRequestGroupMan()->getRequestGroups();
+  for(RequestGroupList::SeqType::const_iterator i = groups.begin(),
+        eoi = groups.end(); i != eoi; ++i) {
     SharedHandle<Dict> entryDict = Dict::g();
     if(requested_key(keys, KEY_STATUS)) {
       entryDict->put(KEY_STATUS, VLB_ACTIVE);
     }
-    gatherProgress(entryDict, *i, e, keys);
+    gatherProgress(entryDict, (*i).second, e, keys);
     list->append(entryDict);
   }
   return list;
 }
 
-const std::deque<SharedHandle<RequestGroup> >&
+const RequestGroupList&
 TellWaitingRpcMethod::getItems(DownloadEngine* e) const
 {
   return e->getRequestGroupMan()->getReservedGroups();
@@ -1046,7 +1073,7 @@ void TellWaitingRpcMethod::createEntry
   gatherProgress(entryDict, item, e, keys);
 }
 
-const std::deque<SharedHandle<DownloadResult> >&
+const DownloadResultList&
 TellStoppedRpcMethod::getItems(DownloadEngine* e) const
 {
   return e->getRequestGroupMan()->getDownloadResults();
@@ -1075,7 +1102,8 @@ SharedHandle<ValueBase> RemoveDownloadResultRpcMethod::process
 
   a2_gid_t gid = str2Gid(gidParam);
   if(!e->getRequestGroupMan()->removeDownloadResult(gid)) {
-    throw DL_ABORT_EX(fmt("Could not remove download result of GID#%" PRId64 "", gid));
+    throw DL_ABORT_EX(fmt("Could not remove download result of GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   return VLB_OK;
 }
@@ -1117,7 +1145,7 @@ void changeOption
   if(option.defined(PREF_DIR) || option.defined(PREF_OUT)) {
     if(dctx->getFileEntries().size() == 1
 #ifdef ENABLE_BITTORRENT
-       && !dctx->hasAttribute(bittorrent::BITTORRENT)
+       && !dctx->hasAttribute(CTX_ATTR_BT)
 #endif // ENABLE_BITTORRENT
        ) {
       dctx->getFirstFileEntry()->setPath
@@ -1127,7 +1155,7 @@ void changeOption
   }
 #ifdef ENABLE_BITTORRENT
   if(option.defined(PREF_DIR) || option.defined(PREF_INDEX_OUT)) {
-    if(dctx->hasAttribute(bittorrent::BITTORRENT)) {
+    if(dctx->hasAttribute(CTX_ATTR_BT)) {
       std::istringstream indexOutIn(grOption->get(PREF_INDEX_OUT));
       std::vector<std::pair<size_t, std::string> > indexPaths =
         util::createIndexPaths(indexOutIn);
@@ -1176,7 +1204,8 @@ SharedHandle<ValueBase> ChangeOptionRpcMethod::process
     }
     changeOption(group, option, e);
   } else {
-    throw DL_ABORT_EX(fmt("Cannot change option for GID#%" PRId64, gid));
+    throw DL_ABORT_EX(fmt("Cannot change option for GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   return VLB_OK;
 }
@@ -1228,11 +1257,10 @@ SharedHandle<ValueBase> GetVersionRpcMethod::process
   SharedHandle<Dict> result = Dict::g();
   result->put(KEY_VERSION, PACKAGE_VERSION);
   SharedHandle<List> featureList = List::g();
-  const FeatureMap& features = FeatureConfig::getInstance()->getFeatures();
-  for(FeatureMap::const_iterator i = features.begin(), eoi = features.end();
-      i != eoi;++i){
-    if((*i).second) {
-      featureList->append((*i).first);
+  for(int feat = 0; feat < MAX_FEATURE; ++feat) {
+    const char* name = strSupportedFeature(feat);
+    if(name) {
+      featureList->append(name);
     }
   }
   result->put(KEY_ENABLED_FEATURES, featureList);
@@ -1247,7 +1275,7 @@ void pushRequestOption
 {
   for(size_t i = 1, len = option::countOption(); i < len; ++i) {
     const Pref* pref = option::i2p(i);
-    const SharedHandle<OptionHandler>& h = oparser->find(pref);
+    const OptionHandler* h = oparser->find(pref);
     if(h && h->getInitialOption() && option->defined(pref)) {
       dict->put(pref->k, option->get(pref));
     }
@@ -1263,7 +1291,8 @@ SharedHandle<ValueBase> GetOptionRpcMethod::process
   a2_gid_t gid = str2Gid(gidParam);
   SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX(fmt("Cannot get option for GID#%" PRId64 "", gid));
+    throw DL_ABORT_EX(fmt("Cannot get option for GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   SharedHandle<Dict> result = Dict::g();
   SharedHandle<Option> option = group->getOption();
@@ -1280,7 +1309,7 @@ SharedHandle<ValueBase> GetGlobalOptionRpcMethod::process
     if(!e->getOption()->defined(pref)) {
       continue;
     }
-    const SharedHandle<OptionHandler>& h = getOptionParser()->find(pref);
+    const OptionHandler* h = getOptionParser()->find(pref);
     if(h) {
       result->put(pref->k, e->getOption()->get(pref));
     }
@@ -1298,13 +1327,13 @@ SharedHandle<ValueBase> ChangePositionRpcMethod::process
   a2_gid_t gid = str2Gid(gidParam);
   int pos = posParam->i();
   const std::string& howStr = howParam->s();
-  RequestGroupMan::HOW how;
+  A2_HOW how;
   if(howStr == "POS_SET") {
-    how = RequestGroupMan::POS_SET;
+    how = A2_POS_SET;
   } else if(howStr == "POS_CUR") {
-    how = RequestGroupMan::POS_CUR;
+    how = A2_POS_CUR;
   } else if(howStr == "POS_END") {
-    how = RequestGroupMan::POS_END;
+    how = A2_POS_END;
   } else {
     throw DL_ABORT_EX("Illegal argument.");
   }
@@ -1330,7 +1359,8 @@ SharedHandle<ValueBase> GetServersRpcMethod::process
   a2_gid_t gid = str2Gid(gidParam);
   SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group || group->getState() != RequestGroup::STATE_ACTIVE) {
-    throw DL_ABORT_EX(fmt("No active download for GID#%" PRId64, gid));
+    throw DL_ABORT_EX(fmt("No active download for GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
   const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
@@ -1376,7 +1406,8 @@ SharedHandle<ValueBase> ChangeUriRpcMethod::process
   size_t index = indexParam->i()-1;
   SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX(fmt("Cannot remove URIs from GID#%" PRId64 "", gid));
+    throw DL_ABORT_EX(fmt("Cannot remove URIs from GID#%s",
+                          GroupId::toHex(gid).c_str()));
   }
   const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
   const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
